/* 
 * Fait par MLL, d'apres la routine quadric.c du package
 * Mesa 3-D graphics library version 2.6 de Brian Paul
*/

#include "mllquad.h"
#include <stdio.h>
#include <math.h>
#ifndef M_PI
#define M_PI	3.14159265358979323846
#endif 

/*
 * Texture coordinates if texture flag is set
 */
#define TXTR_COORD(x,y)    if (qobj->TextureFlag) glTexCoord2f(x,y);

/*
 * Process a GLU error.
 */
static void quadric_error( MLLquadricObj *qobj, GLenum error, const char *msg )
{
   /* Call the error call back function if any */
   if (qobj->ErrorFunc) {
      (*qobj->ErrorFunc)( error );
   }
   /* Print a message to stdout if MESA_DEBUG variable is defined */
   if (getenv("MESA_DEBUG")) {
      fprintf(stderr,"GLUError: %s: %s\n", (char*) gluErrorString(error), msg);
   }
}


MLLquadricObj *mllNewQuadric( void )
{
   MLLquadricObj *q;

   q = (MLLquadricObj *) malloc( sizeof(struct MLLquadricObj) );
   if (q) {
      q->DrawStyle = GLU_FILL;
      q->Orientation = GLU_OUTSIDE;
      q->TextureFlag = GL_FALSE;
      q->Normals = GLU_SMOOTH;
      q->ErrorFunc = NULL;
   }
   return q;
}



void mllDeleteQuadric( MLLquadricObj *state )
{
   if (state) {
      free( (void *) state );
   }
}



/*
 * Set the drawing style to be GLU_FILL, GLU_LINE, GLU_SILHOUETTE,
 * or GLU_POINT.
 */
void mllQuadricDrawStyle( MLLquadricObj *quadObject, GLenum drawStyle )
{
   if (quadObject && (drawStyle==GLU_FILL || drawStyle==GLU_LINE
		   || drawStyle==GLU_SILHOUETTE || drawStyle==GLU_POINT)) {
      quadObject->DrawStyle = drawStyle;
   }
   else {
      quadric_error( quadObject, GLU_INVALID_ENUM, "qluQuadricDrawStyle" );
   }
}



/*
 * Set the orientation to GLU_INSIDE or GLU_OUTSIDE.
 */
void mllQuadricOrientation( MLLquadricObj *quadObject,
                                     GLenum orientation )
{
   if (quadObject && (orientation==GLU_INSIDE || orientation==GLU_OUTSIDE)) {
      quadObject->Orientation = orientation;
   }
   else {
      quadric_error( quadObject, GLU_INVALID_ENUM, "qluQuadricOrientation" );
   }
}



/*
 * Set the error handler callback function.
 */
void mllQuadricCallback( MLLquadricObj *qobj,
                                  GLenum which, void (*fn)(GLenum) )
{
   if (qobj && which==GLU_ERROR) qobj->ErrorFunc = fn;
}


void mllQuadricNormals( MLLquadricObj *quadObject, GLenum normals )
{
   if (quadObject
         && (normals==GLU_NONE || normals==GLU_FLAT || normals==GLU_SMOOTH)) {
      quadObject->Normals = normals;
   }
}


void mllQuadricTexture( MLLquadricObj *quadObject,
                                 GLboolean textureCoords )
{
   if (quadObject) {
      quadObject->TextureFlag = textureCoords;
   }
}

/*
 * Call glNormal3f after scaling normal to unit length.
 */
static void normal3f( GLfloat x, GLfloat y, GLfloat z )
{
   GLdouble mag;

   mag = sqrt( x*x + y*y + z*z );
   if (mag>0.00001F) {
	  x = (GLfloat) (x/mag);
      y = (GLfloat) (y/mag);
      z = (GLfloat) (z/mag);
   glNormal3f(x, y, z);
   }
   else glNormal3f( 0., 0., 1.);
}

/*---------------------------------------------------------------------------------
    Paralllpipde avec ou sans fond et couvercle.
	Les cots sont parallles aux plans du repre courant xyz.
	La face du bas, dans le plan z=0, est trace que si nface == 5 ou 6
	La face du haut, dans le plan z = dz, est trace que si nface == -5 ou 6
	Ces faces sont centres en x et y et ont pour cot dx et dy.
	Les trois valeurs dx, dy et dz doivent tre positives pour que les
	orientations des faces soient correctes.
---------------------------------------------------------------------------------*/
void mllBoite(MLLquadricObj *qobj, GLdouble dx, GLdouble dy, GLdouble dz, int nface)
{
GLfloat x , y, z, zn, nn;


	x = (GLfloat) (dx/2.);
	y = (GLfloat) (dy/2.);
	z = (GLfloat) dz;
	zn = (GLfloat) (dz/2.);

	/* flag direction normale (- si interieur) */
	if (qobj->Orientation == GLU_INSIDE) nn = -1.0;
	else nn = 1.0;

		/* TRACE EN MODE SURFACE */
	if (qobj->DrawStyle == GLU_FILL) 
	{
		if (qobj->TextureFlag) {
		glBegin( GL_QUADS );
			glNormal3f( 0., nn, 0.);
			glTexCoord2f(0., 0.);
			glVertex3f( x, y, z);
			glTexCoord2f(1., 0.);
			glVertex3f( x, y, 0.);
			glTexCoord2f(1., 1.);
			glVertex3f(-x, y, 0.);
			glTexCoord2f(0., 1.);
			glVertex3f(-x, y, z);

			glNormal3f( -nn, 0., 0.);
			glTexCoord2f(0., 0.);
			glVertex3f(-x, y, z);
			glTexCoord2f(0., 1.);
			glVertex3f(-x, y, 0.);
			glTexCoord2f(1., 1.);
			glVertex3f(-x, -y, 0.);
			glTexCoord2f(1., 0.);
			glVertex3f(-x, -y, z);

			glNormal3f( 0., -nn, 0.);
			glTexCoord2f(0., 0.);
			glVertex3f(-x, -y, z);
			glTexCoord2f(0., 1.);
			glVertex3f(-x, -y, 0.);
			glTexCoord2f(1., 1.);
			glVertex3f( x, -y, 0.);
			glTexCoord2f(1., 0.);
			glVertex3f( x, -y, z);

			glNormal3f( nn, 0., 0.);
			glTexCoord2f(0., 0.);
			glVertex3f( x, -y, z);
			glTexCoord2f(0., 1.);
			glVertex3f( x, -y, 0.);
			glTexCoord2f(1., 1.);
			glVertex3f( x, y, 0.);
			glTexCoord2f(1., 0.);
			glVertex3f( x, y, z);
 
			if (nface == 5 || nface == 6) {
			glNormal3f( 0., 0., -nn);
			glTexCoord2f(0., 0.);
			glVertex3f( x, y, 0.);
			glTexCoord2f(0., 1.);
			glVertex3f( x, -y, 0.);
			glTexCoord2f(1., 1.);
			glVertex3f(-x,-y, 0.);
			glTexCoord2f(1., 0.);
			glVertex3f(-x, y, 0.);
			}

			if (nface == -5 || nface == 6) {
			glNormal3f( 0., 0., nn);
			glTexCoord2f(0., 0.);
			glVertex3f( x, y, z);
			glTexCoord2f(0., 1.);
			glVertex3f(-x, y, z);
			glTexCoord2f(1., 1.);
			glVertex3f(-x,-y, z);
			glTexCoord2f(1., 0.);
			glVertex3f( x,-y, z);
			}
		glEnd();
		}
		else
		{
		glBegin( GL_QUADS );
			glNormal3f( 0., nn, 0.);
			glVertex3f( x, y, z);
			glVertex3f( x, y, 0.);
			glVertex3f(-x, y, 0.);
			glVertex3f(-x, y, z);

			glNormal3f( -nn, 0., 0.);
			glVertex3f(-x, y, z);
			glVertex3f(-x, y, 0.);
			glVertex3f(-x, -y, 0.);
			glVertex3f(-x, -y, z);

			glNormal3f( 0., -nn, 0.);
			glVertex3f(-x, -y, z);
			glVertex3f(-x, -y, 0.);
			glVertex3f( x, -y, 0.);
			glVertex3f( x, -y, z);

			glNormal3f( nn, 0., 0.);
			glVertex3f( x, -y, z);
			glVertex3f( x, -y, 0.);
			glVertex3f( x, y, 0.);
			glVertex3f( x, y, z);
 
			if (nface == 5 || nface == 6) {
			glNormal3f( 0., 0., -nn);
			glVertex3f( x, y, 0.);
			glVertex3f( x, -y, 0.);
			glVertex3f(-x,-y, 0.);
			glVertex3f(-x, y, 0.);
			}

			if (nface == -5 || nface == 6) {
			glNormal3f( 0., 0., nn);
			glVertex3f( x, y, z);
			glVertex3f(-x, y, z);
			glVertex3f(-x,-y, z);
			glVertex3f( x,-y, z);
			}
		glEnd();
		}
	}
	/* TRACE EN MODE GLU_POINT */
	else if (qobj->DrawStyle == GLU_POINT) 
	{
		glBegin( GL_POINTS );
			glVertex3f( x, y, 0.);
			glVertex3f(-x, y, 0.);
			glVertex3f(-x,-y, 0.);
			glVertex3f( x,-y, 0.);

			glVertex3f( x, y, z);
			glVertex3f(-x, y, z);
			glVertex3f(-x,-y, z);
			glVertex3f( x,-y, z);
		glEnd();
	}
	/* TRACE EN MODE LIGNE ou SILHOUETTE */
	else if (qobj->DrawStyle==GLU_LINE || qobj->DrawStyle==GLU_SILHOUETTE) 
	{
		glBegin( GL_LINE_LOOP );
			glVertex3f( x, y, 0.);
			glVertex3f(-x, y, 0.);
			glVertex3f(-x,-y, 0.);
			glVertex3f( x,-y, 0.);
		glEnd();
		glBegin( GL_LINE_LOOP );
			glVertex3f( x, y, z);
			glVertex3f(-x, y, z);
			glVertex3f(-x,-y, z);
			glVertex3f( x,-y, z);
		glEnd();
		/* Trace des lignes de cote */
		glBegin( GL_LINES );
			glVertex3f( x, y, 0.);
			glVertex3f( x, y, z);

			glVertex3f(-x, y, 0.);
			glVertex3f(-x, y, z);

			glVertex3f(-x,-y, 0.);
			glVertex3f(-x,-y, z);

			glVertex3f( x,-y, 0.);
			glVertex3f( x,-y, z);
		glEnd();
	}
}

/*---------------------------------------------------------------------------------
    Borne de cots parallles aux plans du repre courant
	delx (>0) paisseur de la borne
	rayy (>0) rayon de la borne qui a pour largeur 2*rayy
	profz (>0) hauteur de la borne = profz+ry
	Le fond de la borne se situe dans le plan z = -profz.
	L'arrondi de la borne qui culmine dansle plan z=rayy comporte nface facettes
	Le fond de la borne est ferm par une face si flagfond != 0.
---------------------------------------------------------------------------------*/
void mllBorne(MLLquadricObj *qobj, GLdouble delx, GLdouble rayy, 
									GLdouble profz, int nface, int flagfond)
{
int i ;
GLfloat x , y, ry, z, hz, nn, ca, sa;
double angle;

	x = (GLfloat) (delx/2.);
	ry = (GLfloat) rayy ;
	hz = (GLfloat) profz ;

	/* flag direction normale (- si interieur) */
	if (qobj->Orientation == GLU_INSIDE) nn = -1.0;
	else nn = 1.0;

		/* TRACE EN MODE SURFACE */
	if (qobj->DrawStyle == GLU_FILL) 
	{
		if (qobj->TextureFlag) {
		GLfloat v1 = hz/(hz+ry), v2=1.f-v1; ;
		glBegin(GL_POLYGON);
			glNormal3f( nn, 0., 0.);
			glTexCoord2f(1., 0.);
		    glVertex3f( x, ry, -hz);
			glTexCoord2f(1., v1);
		    glVertex3f( x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glTexCoord2f(0.5f*(1.f+ca), v1+v2*sa);
				glVertex3f( x, y, z);
			}
			glTexCoord2f(0., v1);
		    glVertex3f(x, -ry, 0.);
			glTexCoord2f(0., 0.);
		    glVertex3f(x, -ry, -hz);
		glEnd();
		glBegin(GL_POLYGON);
			glNormal3f(-nn, 0., 0.);
			glTexCoord2f(1., 0.);
		    glVertex3f(-x, -ry, -hz);
			glTexCoord2f(1., v1);
		    glVertex3f(-x, -ry, 0.);
			for (i=nface-1 ; i > 0  ; i--)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glTexCoord2f(0.5f*(1.f+ca), v1+v2*sa);
				glVertex3f(-x, y, z);
			}
			glTexCoord2f(0., v1);
		    glVertex3f(-x, ry, 0.);
			glTexCoord2f(0., 0.);
		    glVertex3f(-x, ry, -hz);
		glEnd();
		glBegin(GL_QUAD_STRIP);
			glNormal3f( 0., nn, 0.);
			glTexCoord2f(0., 0.);
		    glVertex3f( x, ry, -hz);
			glTexCoord2f(1., 0.);
		    glVertex3f(-x, ry, -hz);
			glTexCoord2f(0., v1);
		    glVertex3f( x, ry, 0.);
			glTexCoord2f(1., v1);
		    glVertex3f(-x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glNormal3f( 0., ca, sa);
				glTexCoord2f(0., v1+v2*sa);
				glVertex3f( x, y, z);
				glTexCoord2f(1., v1+v2*sa);
				glVertex3f(-x, y, z);
			}
			glNormal3f( 0., -nn, 0.);
			glTexCoord2f(0., v1);
		    glVertex3f( x, -ry, 0.);
			glTexCoord2f(1., v1);
		    glVertex3f(-x, -ry, 0.);
			glTexCoord2f(0., 0.);
		    glVertex3f( x, -ry, -hz);
			glTexCoord2f(1., 0.);
		    glVertex3f(-x, -ry, -hz);
		glEnd();
		if (flagfond) 
		{
		glBegin(GL_QUADS);
			glNormal3f( 0., 0., -nn);
		    glVertex3f( x, ry, -hz);
		    glVertex3f( x,-ry, -hz);
		    glVertex3f(-x,-ry, -hz);
		    glVertex3f(-x, ry, -hz);
		glEnd();
		}
		}
		else
		{
		glBegin(GL_POLYGON);
			glNormal3f( nn, 0., 0.);
		    glVertex3f( x, ry, -hz);
		    glVertex3f( x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f( x, y, z);
			}
		    glVertex3f(x, -ry, 0.);
		    glVertex3f(x, -ry, -hz);
		glEnd();
		glBegin(GL_POLYGON);
			glNormal3f(-nn, 0., 0.);
		    glVertex3f(-x, -ry, -hz);
		    glVertex3f(-x, -ry, 0.);
			for (i=nface-1 ; i > 0  ; i--)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f(-x, y, z);
			}
		    glVertex3f(-x, ry, 0.);
		    glVertex3f(-x, ry, -hz);
		glEnd();
		glBegin(GL_QUAD_STRIP);
			glNormal3f( 0., nn, 0.);
		    glVertex3f( x, ry, -hz);
		    glVertex3f(-x, ry, -hz);
		    glVertex3f( x, ry, 0.);
		    glVertex3f(-x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glNormal3f( 0., ca, sa);
				glVertex3f( x, y, z);
				glVertex3f(-x, y, z);
			}
			glNormal3f( 0., -nn, 0.);
		    glVertex3f( x, -ry, 0.);
		    glVertex3f(-x, -ry, 0.);
		    glVertex3f( x, -ry, -hz);
		    glVertex3f(-x, -ry, -hz);
		glEnd();
		if (flagfond) 
		{
		glBegin(GL_QUADS);
			glNormal3f( 0., 0., -nn);
		    glVertex3f( x, ry, -hz);
		    glVertex3f( x,-ry, -hz);
		    glVertex3f(-x,-ry, -hz);
		    glVertex3f(-x, ry, -hz);
		glEnd();
		}
		}
	}
	/* TRACE EN MODE GLU_POINT */
	else if (qobj->DrawStyle == GLU_POINT) 
	{
		glBegin( GL_POINTS );
		    glVertex3f( x, ry, -hz);
		    glVertex3f( x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f( x, y, z);
			}
		    glVertex3f(x, -ry, 0.);
		    glVertex3f(x, -ry, -hz);

		    glVertex3f(-x, -ry, -hz);
		    glVertex3f(-x, -ry, 0.);
			for (i=nface-1 ; i > 0  ; i--)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f(-x, y, z);
			}
		    glVertex3f(-x, ry, 0.);
		    glVertex3f(-x, ry, -hz);
		glEnd();
	}
	/* TRACE EN MODE LIGNE ou SILHOUETTE */
	else if (qobj->DrawStyle==GLU_LINE || qobj->DrawStyle==GLU_SILHOUETTE) 
	{
		glBegin( GL_LINE_LOOP );
		    glVertex3f(x, ry, -hz);
		    glVertex3f(x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f(x, y, z);
			}
		    glVertex3f(x, -ry, 0.);
		    glVertex3f(x, -ry, -hz);
		glEnd();

		glBegin( GL_LINE_LOOP );
		    glVertex3f(-x, -ry, -hz);
		    glVertex3f(-x, -ry, 0.);
			for (i=nface-1 ; i > 0  ; i--)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f(-x, y, z);
			}
		    glVertex3f(-x, ry, 0.);
		    glVertex3f(-x, ry, -hz);
		glEnd();
		/* Trace des lignes de cote */
		glBegin( GL_LINES);
		    glVertex3f( x, ry, -hz);
		    glVertex3f(-x, ry, -hz);
		    glVertex3f( x, ry, 0.);
		    glVertex3f(-x, ry, 0.);
			for (i=1 ; i < nface  ; i++)
			{
				angle = i*M_PI/nface;
				ca = (GLfloat) cos(angle) ; 
				sa = (GLfloat) sin(angle) ; 
				y = ry*ca;
				z = ry*sa;
				glVertex3f( x, y, z);
				glVertex3f(-x, y, z);
			}
		    glVertex3f( x, -ry, 0.);
		    glVertex3f(-x, -ry, 0.);
		    glVertex3f( x, -ry, -hz);
		    glVertex3f(-x, -ry, -hz);
		glEnd();
	}
}
