#ifndef MLLQUAQ_H
#define MLLQUAD_H
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>

typedef struct MLLquadricObj {
	GLenum	DrawStyle;		/* GLU_FILL, LINE, SILHOUETTE, or POINT */
	GLenum Orientation;		/* GLU_INSIDE or GLU_OUTSIDE */
	GLboolean TextureFlag;	/* Generate texture coords? */
	GLenum Normals;			/* GLU_NONE, GLU_FLAT, or GLU_SMOOTH */
	void (*ErrorFunc)(GLenum err);	/* Error handler callback function */
} MLLquadricObj;

MLLquadricObj *mllNewQuadric(void);

void mllDeleteQuadric(MLLquadricObj *state);

void mllQuadricDrawStyle(MLLquadricObj *quadObject, GLenum drawStyle);

void mllQuadricOrientation(MLLquadricObj *quadObject, GLenum orientation);

void mllQuadricCallback(MLLquadricObj *qobj, GLenum which, void (*fn)(GLenum));

void mllQuadricNormals(MLLquadricObj *quadObject, GLenum normals);

void mllQuadricTexture(MLLquadricObj *quadObject, GLboolean textureCoords );

/*---------------------------------------------------------------------------------
    Paralllpipde avec ou sans fond et couvercle.
	Les cots sont parallles aux plans du repre courant xyz.
	La face du bas, dans le plan z=0, est trace que si nface == 5 ou 6
	La face du haut, dans le plan z = dz, est trace que si nface == -5 ou 6
	Ces faces sont centres en x et y et ont pour cot dx et dy.
	Les trois valeurs dx, dy et dz doivent tre positives pour que les
	orientations des faces soient correctes.
---------------------------------------------------------------------------------*/
void mllBoite(MLLquadricObj *qobj, GLdouble dx, GLdouble dy, GLdouble dz, int nface) ;

/*---------------------------------------------------------------------------------
    Borne de cots parallles aux plans du repre courant
	delx (>0) paisseur de la borne
	rayy (>0) rayon de la borne qui a pour largeur 2*rayy
	profz (>0) hauteur de la borne = profz+ry
	Le fond de la borne se situe dans le plan z = -profz.
	L'arrondi de la borne qui culmine dansle plan z=rayy comporte nface facettes
	Le fond de la borne est ferm par une face si flagfond != 0.
---------------------------------------------------------------------------------*/
void mllBorne(MLLquadricObj *qobj, GLdouble delx, GLdouble rayy, 
									GLdouble profz, int nface, int flagfond) ;
#endif