#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef M_PI
#define M_PI 3.14159265358979
#endif
#include "robutils.h"

//#define VERBEUX

/* Liberation allocations internes structure robot */
void freeRobot(UnRobot *rob)
{
	if (rob == NULL) return ;
	if (rob->ri != NULL) free(rob->ri) ;
	if (rob->ui != NULL) free(rob->ui) ;
	if (rob->li != NULL) free(rob->li) ;
	rob->ri = NULL;
	rob->ui = NULL;
	rob->li = NULL;
}

/*-------------------------------------------------------------
	Lectures paramtres gomtriques robot, a savoir :
	type d'axe, composantes Li-1i, azi et sit Ui.
	et allocation des structures internes de stockage
--------------------------------------------------------------*/
int LitRobot(char *nomrob, UnRobot *rob)
{
FILE *lu ;
int i, n, N, typ ;
double x,y,z,azi,sit,csit;

	i = sizeof(double) ;
	n = sizeof(Vec3);

	lu = fopen(nomrob,"rt");
	if (lu == NULL) return 0;
	rob->nq = n = 0 ;
	rob->ri = NULL;
	rob->ui = NULL;
	rob->li = NULL;
	do
	{
		i = fscanf(lu,"%d",&typ);
		if (i != 1 || (typ != 0 && typ != 1)) break ;
		i = fscanf (lu, "%lf %lf %lf %lf %lf",&x, &y, &z, &azi, &sit);
		if (i == 5)
		{
			sit *= M_PI/180. ; azi *= M_PI/180.;
			rob->nq++;
			rob->ri = realloc(rob->ri, rob->nq*sizeof(int));
			rob->ui = realloc(rob->ui, rob->nq*sizeof(Vec3));
			rob->li = realloc(rob->li, rob->nq*sizeof(Vec3));
			rob->ri[n] = typ ;
			rob->li[n][0] = x ; 
			rob->li[n][1] = y ; 
			rob->li[n][2] = z ; 
			csit = cos(sit) ;
			rob->ui[n][0] = cos(azi)*csit ; 
			rob->ui[n][1] = sin(azi)*csit ; 
			rob->ui[n][2] = sin(sit) ; 
			n = rob->nq ;
		}
	} while (1);
	N = rob->nq ;
#ifdef VERBEUX
	printf("  Axe  ");  for(n=0;n<N;n++)printf("%3d    ",n) ; 
	printf("\n");
	printf("\nRho(i) ");for(n=0;n<N;n++)printf("%3d    ",rob->ri[n]) ; 
	printf("\n");
	printf("\n       ");for(n=0;n<N;n++)printf("%7.3f",rob->ui[n][0]) ; 
	printf("\n u(i)  ");for(n=0;n<N;n++)printf("%7.3f",rob->ui[n][1]) ; 
	printf("\n       ");for(n=0;n<N;n++)printf("%7.3f",rob->ui[n][2]) ; 
	printf("\n");
	printf("\n       ");for(n=0;n<N;n++)printf("%7.3f",rob->li[n][0]) ; 
	printf("\n l(i)  ");for(n=0;n<N;n++)printf("%7.3f",rob->li[n][1]) ; 
	printf("\n       ");for(n=0;n<N;n++)printf("%7.3f",rob->li[n][2]) ;
	printf("\n");
#endif
	return N ;
}

/*------------------------------------------------------------------
      Calcul modle gomtrique Dop et jacobienne J dans Rp 
          du robot rob, pour la configuration qi           
--------------------------------------------------------------------*/
void qi2DopetJp(UnRobot *rob, double *qi, double *Dop, double *Jacp)
{
int i ;
double *Jv, *Jw;
Por Dij;

	eyem3(Dop+3); setv3(Dop,0.) ;
	for (i = rob->nq-1 ; i >= 0; i--) 
	{
		Jv = &Jacp[i*6] ; Jw = Jv+3 ;
		if (rob->ri[i]) /* Cas rotation */
		{
			cpyv3(Dij, rob->li[i]);
			av3tomrot(Dij+3, qi[i], rob->ui[i]);	
			
			crossv3(Jv, rob->ui[i], Dop);
			cpyv3(Jw, rob->ui[i]);
		}
		else /* Cas translation */
		{
			cumv3(Dij, rob->li[i], qi[i], rob->li[i]);
			eyem3(Dij+3);

			cpyv3(Jv, rob->ui[i]);
			setv3(Jw, 0.);
		}
		tm3v3 (Jv, Dop+3, Jv) ;
		tm3v3 (Jw, Dop+3, Jw) ;
		porpor(Dop,Dij,Dop) ;
	}

}

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dop
	    dans la mme nappe que le qi initial 
		(en theorie, sauf franchissement accidentel point singulier)
	 Renvoi :
	 = nombre d'itrations >= 0 si convergence
	 = -1 si divergence
	 = -2 si nombre max d'itrations atteint
	 = -3 si singularit rsolution
------------------------------------------------------------------------*/
int Dop2qi(UnRobot *rob, double *Dop, double *qi, int maxiter,
		double epsang, double epspos, double dalim, double dplim, double *dqlim)
{
double Jacp[6*NQMAX], JJ[36] ;
Por Dcou;
double Erpos[6], *Eratt = Erpos+3;
double Delq[NQMAX], coefs[NQMAX], cmax ;
Mat3 E;
int i, k = 0, N = rob->nq ;
double ang, dist;
double oldang = 1.e+100, olddist = 1.e+100;

	if (N > NQMAX)
	{
		printf("NQMAX limit  10. Recompiler pour augmenter sa valeur !");
		exit(1) ;
	}
	if (N < 6)
	{
		printf("L'inversion suppose au moins 6 axes !");
		exit(1) ;
	}
	do
	{
		/* Situation courante */
		qi2DopetJp(rob, qi, Dcou, Jacp) ;
		/* Erreur de position */
		subv3(Erpos, Dop, Dcou);
		tm3v3(Erpos, Dop+3, Erpos) ; /* Dans Rp */
		dist = longv3(Erpos) ;
		/* Erreur d'orientation dans Rp */
		tm3m3(E, Dcou+3, Dop+3);
		Eratt[0] = (E[5] - E[7])/2. ;
		Eratt[1] = (E[6] - E[2])/2. ;
		Eratt[2] = (E[1] - E[3])/2. ;
		/* On confond ang avec sin(ang), satur  1 si cos(ang) < 0. */
		if( E[0]+E[4]+E[8] < 1.) { ang = 1. ; lunitv3(Eratt, Eratt); }
		/* Supprime car precision inutile .........
		else { ang = asin(lunitv3(Eratt, Eratt)); scav3(Eratt, ang, Eratt);} */
		else { ang = longv3(Eratt);}
		/* test si convergence */
		if (ang < epsang && dist < epspos) return k ;
		/* test si divergence */
		//if (ang > oldang || dist > olddist) return -1;
		oldang = ang ; olddist = dist;
		/* saturations homothtiques */
		if (dist > dplim) scav3(Erpos, dplim/dist, Erpos) ;
		if (ang > dalim) scav3(Eratt, dalim/ang, Eratt) ;
		if (N > 6) /* Cas redondant */
		{
			mattmat (JJ, Jacp, Jacp, 6, N, 6) ; /* J*J'*/
			i = solvmat (Erpos, JJ, Erpos, 6, 1, 0) ;
			if (i==0) return -3 ;
			tmatmat (Delq, Jacp, Erpos, N, 6, 1) ;
		}
		else if (N == 6) /* Cas standard */
		{
			i = solvmat (Delq, Jacp, Erpos, 6, 1, 0) ;
			if (i==0) return -3 ;
		}
		else /* Cas sous-dtermin */
		{
			tmatmat (Delq, Jacp, Erpos, N, 6, 1) ;
			tmatmat (JJ, Jacp, Jacp, N, 6, N) ; /* J'*J*/
			i = solvmat (Delq, JJ, Delq, N, 1, 0) ;
			if (i==0) return -3 ;
			addvec (qi, qi, Delq, N) ;
		}
		/* Saturation homothtique du Deltaq */
		divvec(coefs,Delq,dqlim,N);
		cmax = xmaxvec(absvec(coefs, coefs,N),N) ;
		if (cmax > 1.) scavec(Delq,1./cmax,Delq,N);
		addvec (qi, qi, Delq, N) ;

		k++;
		if (k > maxiter) return -2 ;
	} while (1) ;
	return k; /* bidon */
}


